
;**************************************************************************
;*                             TTL 74xx tester                            *
;*                            CMOS 40xx tester                            *
;**************************************************************************
;**************************************************************************
;*          v 1.20 - 21.08.2011                                           *
;**************************************************************************
;*          Hardw.: PIC16F874 used                                        *
;*          OSC.......: Internal osc. used     POWER.....:  18V extern.   *
;**************************************************************************



 INCLUDE "P16F724.INC"


	__CONFIG	_CONFIG1, _DEBUG_OFF & _PLL_EN & _BORV_2_5 & _BOR_OFF & _CP_OFF & _MCLR_DIS & _PWRT_DIS & _WDT_OFF & _INTRC_OSC_NOCLKOUT
	__CONFIG	_CONFIG2, _VCAP_DIS


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"
disp1		EQU	H'22'		;first digit
disp2		EQU	H'23'		;second digit
LED1		EQU	H'24'
LED2		EQU	H'25'
LED3		EQU	H'26'
LED4		EQU	H'27'

;interrupt
W_TEMP		EQU	H'28'
STATUS_TEMP	EQU	H'29'
PCLATH_TEMP	EQU	H'2A'
;
dled		EQU	H'2B'
dd12		EQU	H'2C'
d_digit		EQU	H'2D'		;code of the digit (0 - 9; - )
d2d		EQU	H'2E'		;digit to decode
;
gate1		EQU	H'2F'
gate2		EQU	H'30'
gate3		EQU	H'31'
gate4		EQU	H'32'
gate5		EQU	H'33'
gate6		EQU	H'34'
;
disp3		EQU	H'35'		;3rd digit (info)
;
ata		EQU	H'36'		;auto test
ata_err		EQU	H'37'
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	RESET		;goto RESET routine when boot.

		ORG	004

;save important variables
		MOVWF W_TEMP		;Copy W to W_TEMP register
		SWAPF STATUS,W		;Swap status to be saved into W
;Swaps are used because they do not affect the status bits
		BANKSEL STATUS_TEMP	;Select regardless of current bank
		MOVWF STATUS_TEMP	;Copy status to bank zero STATUS_TEMP register
		MOVF PCLATH,W		;Copy PCLATH to W register
		MOVWF PCLATH_TEMP	;Copy W register to PCLATH_TEMP

		bcf	INTCON,GIE

		PAGESEL	PAGE0

		goto	int_serv


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

RESET		nop
PAGE0
;
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'11011111'	;Set PIC options (See datasheet).
		MOVWF	OPTION_REG	;Write the OPTION register.
		bcf	OPTION_REG,T0CS	;set-up TIMER0 as timer
		bsf	OPTION_REG,PSA	;prescaler NOT for TIMER0
		;
		CLRF	INTCON		;Disable interrupts

;RA0 - RA5: pin13 - 8
;RA6      : not connected
;RA7      : "S2"

;RB0 - RB5: pin 1 - 7
;RB6      : "run"
;RB7      : power control

;RC0 - RC7: duo LED's

;RD0 - RD7: anody  7-seg.

;RE0 - RE1: katody 7-seg.
;RE2      : not connected
;RE3      : "S1"


		MOVLW	B'11111111'	;PORTA = input
		MOVWF	TRISA

		MOVLW	B'01111111'	;PORTB = input/output
		MOVWF	TRISB

		MOVLW	B'00000000'	;PORTC = output
		MOVWF	TRISC

		MOVLW	B'00000000'	;PORTD = output - 7-seg.disp.
		MOVWF	TRISD

		MOVLW	B'00001000'	;PORTE = output; RE3 = input
		MOVWF	TRISE


		bsf	STATUS,RP0
		bsf	STATUS,RP1
		clrf	ANSELB
		clrf	ANSELA

		BCF	STATUS,RP0	;Switch Back to reg. Bank 0
		bcf	STATUS,RP1

; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXX       initialization  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
		call	hw_test


		bcf	PORTB,D'007'	;switch the power off
		clrf	LED1
		clrf	LED2
		clrf	LED3
		clrf	LED4
		movlw	D'010'		;display '--'
		movwf	disp1
		movwf	disp2

		movlw	D'022'		;display "A"
		movwf	disp3
		
		clrf	dd12		;displaying first digit at first (for interrupt)
		clrf	TMR0
		bsf	INTCON,GIE	;enable interrupts
		bsf	INTCON,T0IE	;enable TIMER0 overflow interrupt

		btfss	PORTB,D'006'
		call	fprs		;run switch needs to be off first

		clrf	LED1
		clrf	LED2
		clrf	LED3
		clrf	LED4

		clrf	ata
;-------------------------------------------------------------------------------
;--- init phase is finished now ------------------------------------------------
;-------------------------------------------------------------------------------
xauto		btfsc	PORTE,D'003'
		goto	xauto
cauto		btfss	PORTE,D'003'
		goto	cauto
		call	auto_test


x7400		btfsc	PORTE,D'003'
		goto	x7400
;"up" was pressed
c7400		btfss	PORTE,D'003'
		goto	c7400
		call	TTL7400

x7410		btfsc	PORTE,D'003'
		goto	x7410
c7410		btfss	PORTE,D'003'
		goto	c7410
		call	TTL7410

x7420		btfsc	PORTE,D'003'
		goto	x7420
c7420		btfss	PORTE,D'003'
		goto	c7420
		call	TTL7420

x7430		btfsc	PORTE,D'003'
		goto	x7430
c7430		btfss	PORTE,D'003'
		goto	c7430
		call	TTL7430

x7440		btfsc	PORTE,D'003'
		goto	x7440
c7440		btfss	PORTE,D'003'
		goto	c7440
		call	TTL7440

x4001		btfsc	PORTE,D'003'
		goto	x4001
c4001		btfss	PORTE,D'003'
		goto	c4001
		PAGESEL	PAGE1
		call	CM4001
		PAGESEL	PAGE0

x4002		btfsc	PORTE,D'003'
		goto	x4002
c4002		btfss	PORTE,D'003'
		goto	c4002
		PAGESEL	PAGE1
		call	CM4002
		PAGESEL	PAGE0

x4011		btfsc	PORTE,D'003'
		goto	x4011
c4011		btfss	PORTE,D'003'
		goto	c4011
		PAGESEL	PAGE1
		call	CM4011
		PAGESEL	PAGE0

x4012		btfsc	PORTE,D'003'
		goto	x4012
c4012		btfss	PORTE,D'003'
		goto	c4012
		PAGESEL	PAGE1
		call	CM4012
		PAGESEL	PAGE0

x4025		btfsc	PORTE,D'003'
		goto	x4025
c4025		btfss	PORTE,D'003'
		goto	c4025
		PAGESEL	PAGE1
		call	CM4025
		PAGESEL	PAGE0

x4030		btfsc	PORTE,D'003'
		goto	x4030
c4030		btfss	PORTE,D'003'
		goto	c4030
		PAGESEL	PAGE1
		call	CM4030
		PAGESEL	PAGE0

x4068		btfsc	PORTE,D'003'
		goto	x4068
c4068		btfss	PORTE,D'003'
		goto	c4068
		PAGESEL	PAGE1
		call	CM4068
		PAGESEL	PAGE0

x4071		btfsc	PORTE,D'003'
		goto	x4071
c4071		btfss	PORTE,D'003'
		goto	c4071
		PAGESEL	PAGE1
		call	CM4071
		PAGESEL	PAGE0

x4072		btfsc	PORTE,D'003'
		goto	x4072
c4072		btfss	PORTE,D'003'
		goto	c4072
		PAGESEL	PAGE1
		call	CM4072
		PAGESEL	PAGE0

x4075		btfsc	PORTE,D'003'
		goto	x4075
c4075		btfss	PORTE,D'003'
		goto	c4075
		PAGESEL	PAGE1
		call	CM4075
		PAGESEL	PAGE0

x4081		btfsc	PORTE,D'003'
		goto	x4081
c4081		btfss	PORTE,D'003'
		goto	c4081
		PAGESEL	PAGE1
		call	CM4081
		PAGESEL	PAGE0

x4082		btfsc	PORTE,D'003'
		goto	x4082
c4082		btfss	PORTE,D'003'
		goto	c4082
		PAGESEL	PAGE1
		call	CM4082
		PAGESEL	PAGE0

		goto	xauto


;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 Auto Test
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;following ICs can be detected automatically:
;7400
;7410
;7420 (eq to 7440)
;7430
;
;4001
;4002
;4011
;4012
;4025
;4030
;4068 !!!
;4071
;4072
;4075
;4081
;4082

auto_test	movlw	D'010'
		movwf	disp1
		movlw	D'010'
		movwf	disp2
		movlw	D'022'
		movwf	disp3			;"A" - auto

		clrf	LED1
		clrf	LED2
		clrf	LED3
		clrf	LED4
;
yauto		btfss	PORTE,D'003'
		goto	yauto
;run requested?
		btfss	PORTB,D'006'
		goto	runauto

		clrf	ata

;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	yauto
;-------------------------------------------
runauto		bsf	ata,D'000'
		clrf	ata_err
		call	TTL7400
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 7400
;
		clrf	ata_err
		call	TTL7410
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 7410
;
		clrf	ata_err
		call	TTL7420
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 7420
;
		clrf	ata_err
		call	TTL7430
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 7430
;
		clrf	ata_err
		call	TTL7440
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 7440
;
		clrf	ata_err
		PAGESEL	PAGE1
		call	CM4001
		PAGESEL	PAGE0
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 4001
;
		clrf	ata_err
		PAGESEL	PAGE1
		call	CM4002
		PAGESEL	PAGE0
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 4002
;
		clrf	ata_err
		PAGESEL	PAGE1
		call	CM4011
		PAGESEL	PAGE0
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 4011
;
		clrf	ata_err
		PAGESEL	PAGE1
		call	CM4012
		PAGESEL	PAGE0
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 4012
;
		clrf	ata_err
		PAGESEL	PAGE1
		call	CM4025
		PAGESEL	PAGE0
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 4025
;
		clrf	ata_err
		PAGESEL	PAGE1
		call	CM4030
		PAGESEL	PAGE0
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 4030
;
		clrf	ata_err
		PAGESEL	PAGE1
		call	CM4068
		PAGESEL	PAGE0
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 4068
;
		clrf	ata_err
		PAGESEL	PAGE1
		call	CM4071
		PAGESEL	PAGE0
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 4071
;
		clrf	ata_err
		PAGESEL	PAGE1
		call	CM4072
		PAGESEL	PAGE0
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 4072
;
		clrf	ata_err
		PAGESEL	PAGE1
		call	CM4075
		PAGESEL	PAGE0
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 4075
;
		clrf	ata_err
		PAGESEL	PAGE1
		call	CM4081
		PAGESEL	PAGE0
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 4081
;
		clrf	ata_err
		PAGESEL	PAGE1
		call	CM4082
		PAGESEL	PAGE0
		btfss	ata_err,D'000'
		goto	ata_fin			;finish - 4082
;...
;nothing found

		clrf	LED1
		clrf	LED2
		clrf	LED3
		movlw	B'00000001'
		movwf	LED4			;switch the last LED to red


		movlw	D'025'			;E
		movwf	disp1
		movlw	D'026'			;r
		movwf	disp2
		movlw	D'022'
		movwf	disp3			;"A" - auto

ata_fin		call	power_off

;----
ata_wait	btfss	PORTB,D'006'
		goto	ata_wait
;
		goto	auto_test
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 TTL 7400
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;first implemented 
TTL7400		movlw	D'000'
		movwf	disp1
		movlw	D'000'
		movwf	disp2
		movlw	D'020'
		movwf	disp3			;"L" - TTL
;4 gates --> all 4 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		movwf	LED3
		movwf	LED4
;
y7400		btfss	PORTE,D'003'
		goto	y7400
;run requested?
		btfss	PORTB,D'006'
		goto	run7400
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y7400
;-------------------------------------------
run7400
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01100100'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11100100'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0

;2. switch on power
		bsf	PORTB,D'007'

;3. execute test
		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'
		bsf	gate3,D'007'
		bsf	gate4,D'007'
test7400

;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1

;Gate #1/2
		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		btfsc	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'
;Gate #2/1
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bcf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bsf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfsc	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2

;Gate #3
		bcf	gate3,D'000'
;Gate #3/1
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/2
		bcf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/3
		bsf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/4
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfsc	PORTA,D'005'
		bsf	gate3,D'000'		;error on gate 3


;Gate #4
		bcf	gate4,D'000'
;Gate #4/1
		bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/2
		bcf	PORTA,D'000'
		bsf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/3
		bsf	PORTA,D'000'
		bcf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/4
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		btfsc	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4

		call	LED_set

;----
		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test7400
;
		call	power_off

		goto	TTL7400
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------


;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 TTL 7410
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

TTL7410		movlw	D'001'
		movwf	disp1
		movlw	D'000'
		movwf	disp2
		movlw	D'020'
		movwf	disp3			;"L" - TTL
;3 gates --> 3 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		movwf	LED3
		clrf	LED4
;
y7410		btfss	PORTE,D'003'
		goto	y7410
;run requested?
		btfss	PORTB,D'006'
		goto	run7410
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y7410
;-------------------------------------------
run7410
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01100000'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11100010'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0

;2. switch on power
		bsf	PORTB,D'007'

;3. execute test
		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'
		bsf	gate3,D'007'
test7410

;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		bcf	PORTA,D'000'
		btfss	PORTA,D'001'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTA,D'000'
		btfss	PORTA,D'001'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		bsf	PORTA,D'000'
		btfss	PORTA,D'001'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bcf	PORTA,D'000'
		btfss	PORTA,D'001'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/5
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTA,D'000'
		btfsc	PORTA,D'001'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'

;Gate #2/1
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bcf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bsf	PORTB,D'002'
		bcf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/5
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfsc	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2

;Gate #3
		bcf	gate3,D'000'

;Gate #3/1
		bcf	PORTA,D'002'
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/2
		bcf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/3
		bsf	PORTA,D'002'
		bcf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/4
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/5
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfsc	PORTA,D'005'
		bsf	gate3,D'000'		;error on gate 3


		call	LED_set

;----

		btfsc	ata,D'000'		;auto test active?
		return				;yes - return


		btfss	PORTB,D'006'
		goto	test7410
;
		call	power_off

		goto	TTL7410
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------


;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 TTL 7420
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

TTL7420		movlw	D'002'
		movwf	disp1
		movlw	D'000'
		movwf	disp2
		movlw	D'020'
		movwf	disp3			;"L" - TTL
;2 gates --> 2 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		clrf	LED3
		clrf	LED4
;
y7420		btfss	PORTE,D'003'
		goto	y7420
;run requested?
		btfss	PORTB,D'006'
		goto	run7420
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y7420
;-------------------------------------------
run7420
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01100100'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11100100'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0

;2. switch on power
		bsf	PORTB,D'007'

;3. execute test
		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'
test7420

;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bcf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/5
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/6
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfsc	PORTB,D'005'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'

;Gate #2/1
		bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bcf	PORTA,D'000'
		bsf	PORTA,D'001'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bsf	PORTA,D'000'
		bcf	PORTA,D'001'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		bcf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/5
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		bsf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/6
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfsc	PORTA,D'005'
		bsf	gate2,D'000'		;error on gate 2


		call	LED_set

;----

		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test7420
;
		call	power_off

		goto	TTL7420
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 TTL 7430
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
TTL7430		movlw	D'003'
		movwf	disp1
		movlw	D'000'
		movwf	disp2
		movlw	D'020'
		movwf	disp3			;"L" - TTL
;oen gate --> first LED is used
		movlw	B'00000001'
		movwf	LED1
		clrf	LED2
		clrf	LED3
		clrf	LED4
;
y7430		btfss	PORTE,D'003'
		goto	y7430
;run requested?
		btfss	PORTB,D'006'
		goto	run7430
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y7430
;-------------------------------------------
run7430
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01000000'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11111001'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'

test7430	nop

;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		bcf	PORTB,D'005'
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		btfss	PORTA,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTB,D'005'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		btfss	PORTA,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTB,D'005'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		btfss	PORTA,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bcf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTB,D'005'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		btfss	PORTA,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/5
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bcf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTB,D'005'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		btfss	PORTA,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/6
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bcf	PORTB,D'004'
		bsf	PORTB,D'005'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		btfss	PORTA,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/7
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bcf	PORTB,D'005'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		btfss	PORTA,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/8
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTB,D'005'
		bcf	PORTA,D'001'
		bsf	PORTA,D'002'
		btfss	PORTA,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/9
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTB,D'005'
		bsf	PORTA,D'001'
		bcf	PORTA,D'002'
		btfss	PORTA,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/10
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTB,D'005'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		btfsc	PORTA,D'005'
		bsf	gate1,D'000'		;error on gate 1


		call	LED_set
;

		btfsc	ata,D'000'		;auto test active?
		return				;yes - return


		btfss	PORTB,D'006'
		goto	test7430
		call	power_off
		goto	TTL7430
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 TTL 7440
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
TTL7440		movlw	D'004'
		movwf	disp1
		movlw	D'000'
		movwf	disp2
		movlw	D'020'
		movwf	disp3			;"L" - TTL
;two gates --> first 2 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		clrf	LED3
		clrf	LED4
;
y7440		btfss	PORTE,D'003'
		goto	y7440
;run requested?
		btfss	PORTB,D'006'
		goto	run7440
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y7440
;-------------------------------------------
run7440
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01100100'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11100100'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'

test7440	nop

;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bcf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/5
		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/6
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfsc	PORTB,D'005'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'
;Gate #2/1
		bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		bsf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		bcf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bsf	PORTA,D'000'
		bcf	PORTA,D'001'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/5
		bcf	PORTA,D'000'
		bsf	PORTA,D'001'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/6
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfsc	PORTA,D'005'
		bsf	gate2,D'000'		;error on gate 2

		call	LED_set
;

		btfsc	ata,D'000'		;auto test active?
		return				;yes - return


		btfss	PORTB,D'006'
		goto	test7440
		call	power_off
		goto	TTL7440
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------




;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;some service routines....
;-------------------------------------------------------------------------------
power_off	BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'11111111'	;PORTA = input
		MOVWF	TRISA
		MOVLW	B'01111111'	;PORTB = input/output
		MOVWF	TRISB
		BCF	STATUS,RP0	;Switch to register bank 0
;
		bcf	PORTB,D'007'	;switch the power off
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;LED set based on info in gatex variables:
;	bit7 = 1 - gate is used
;	bit0 = 1 - there was an error in current test cycle
;	bit1 = 1 - there was an error during current test

LED_set		btfsc	gate1,D'007'
		call	set_LED1
		btfsc	gate2,D'007'
		call	set_LED2
		btfsc	gate3,D'007'
		call	set_LED3
		btfsc	gate4,D'007'
		call	set_LED4

		return
;--------------------------------------------------
set_LED1	btfsc	gate1,D'001'
		goto	g1_err
		btfsc	gate1,D'000'
		goto	g1_err
;g1_ok
		movlw	B'00000010'		;green light
		movwf	LED1
		return
;
g1_err		movlw	B'00000001'		;red light
		movwf	LED1
		bsf	gate1,D'001'
		bsf	ata_err,D'000'
		return
;----------------------------
set_LED2	btfsc	gate2,D'001'
		goto	g2_err
		btfsc	gate2,D'000'
		goto	g2_err
;g2_ok
		movlw	B'00000010'		;green light
		movwf	LED2
		return
;
g2_err		movlw	B'00000001'		;red light
		movwf	LED2
		bsf	gate2,D'001'
		bsf	ata_err,D'000'
		return
;----------------------------
set_LED3	btfsc	gate3,D'001'
		goto	g3_err
		btfsc	gate3,D'000'
		goto	g3_err
;g3_ok
		movlw	B'00000010'		;green light
		movwf	LED3
		return
;
g3_err		movlw	B'00000001'		;red light
		movwf	LED3
		bsf	gate3,D'001'
		bsf	ata_err,D'000'
		return
;----------------------------
set_LED4	btfsc	gate4,D'001'
		goto	g4_err
		btfsc	gate4,D'000'
		goto	g4_err
;g4_ok
		movlw	B'00000010'		;green light
		movwf	LED4
		return
;
g4_err		movlw	B'00000001'		;red light
		movwf	LED4
		bsf	gate4,D'001'
		bsf	ata_err,D'000'
		return
;-------------------------------------------------------------------------------
;wait for switching off the "run" request
fprs		movlw	B'00000001'		;red light on for LED1 and LED2
		movwf	LED1
		movwf	LED2
		clrf	LED3
		clrf	LED4

		MOVLW   D'255'
		MOVWF   TIMER2
fdl1		MOVLW   D'255'
		MOVWF   TIMER1

fdl2		btfsc	PORTB,D'006'
		return				;run request cancelled...
		DECFSZ  TIMER1,F
		GOTO    fdl2
		DECFSZ  TIMER2,F
		GOTO    fdl1

		movlw	B'00000001'		;red light on for LED3 and LED4
		movwf	LED3
		movwf	LED4
		clrf	LED1
		clrf	LED2

		MOVLW   D'255'
		MOVWF   TIMER2
fdl3		MOVLW   D'255'
		MOVWF   TIMER1

fdl4		btfsc	PORTB,D'006'
		return				;run request cancelled...
		DECFSZ  TIMER1,F
		GOTO    fdl4
		DECFSZ  TIMER2,F
		GOTO    fdl3

		goto	fprs


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x             interrupt service routine                              x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
;TIMER2 input is int.osc./4


int_serv	nop

;business logic... ---------------------------------------------------------------------
;duo LED display
lxdc1
		clrf	dled

		btfsc	LED1,D'000'
		call	led1_red
		btfsc	LED1,D'001'
		call	led1_green

		btfsc	LED2,D'000'
		call	led2_red
		btfsc	LED2,D'001'
		call	led2_green

		btfsc	LED3,D'000'
		call	led3_red
		btfsc	LED3,D'001'
		call	led3_green

		btfsc	LED4,D'000'
		call	led4_red
		btfsc	LED4,D'001'
		call	led4_green

		movf	dled,0
		movwf	PORTC

;7-seg.display
		btfss	dd12,D'000'
		goto	disp_first
		btfss	dd12,D'001'
		goto	disp_third
;-----------------------------------------------------------
;display second digit
		movf	disp2,0
		movwf	d2d
		call	decode_digit
		movlw	B'00000111'		;switch all digits off
		movwf	PORTE
		movf	d_digit,0
		movwf	PORTD
		movlw	B'00000101'		;second digit
		movwf	PORTE

		clrf	dd12
		goto	ir_cnt2
;-----------------------------------------------------------
;display first digit
disp_first
		movf	disp1,0
		movwf	d2d
		call	decode_digit
		movlw	B'00000111'		;switch both digits off
		movwf	PORTE
		movf	d_digit,0
		movwf	PORTD
		movlw	B'00000110'		;first digit
		movwf	PORTE

		bsf	dd12,D'000'
		goto	ir_cnt2

;-----------------------------------------------------------
;display thirs digit
disp_third
		movf	disp3,0
		movwf	d2d
		call	decode_digit
		movlw	B'00000111'		;switch both digits off
		movwf	PORTE
		movf	d_digit,0
		movwf	PORTD
		movlw	B'00000011'		;third digit
		movwf	PORTE

		clrf	dd12
		bsf	dd12,D'001'

ir_cnt2
;---------------------------------------------------------------------------------------
		clrf	TMR0
;
;restore important variables
		BANKSEL STATUS_TEMP	;Select regardless of current bank
		MOVF PCLATH_TEMP,W	;
		MOVWF PCLATH		;Restore PCLATH
		SWAPF STATUS_TEMP,W	;Swap STATUS_TEMP register into W
					;(sets bank to original state)
		MOVWF STATUS		;Move W into STATUS register
		SWAPF W_TEMP,F		;Swap W_TEMP
		SWAPF W_TEMP,W		;Swap W_TEMP into W

		bcf	INTCON,T0IF
		bsf	INTCON,GIE
		bsf	INTCON,T0IE
		retfie
;---------------------------------------------------------------------------------------
;------------------------------------------------
led1_green	bsf	dled,D'000'
		bcf	dled,D'001'
		return
led1_red	bsf	dled,D'001'
		bcf	dled,D'000'
		return
;------------------------------------------------
led2_green	bsf	dled,D'002'
		bcf	dled,D'003'
		return
led2_red	bcf	dled,D'002'
		bsf	dled,D'003'
		return
;------------------------------------------------
led3_green	bsf	dled,D'004'
		bcf	dled,D'005'
		return
led3_red	bcf	dled,D'004'
		bsf	dled,D'005'
		return
;------------------------------------------------
led4_green	bsf	dled,D'006'
		bcf	dled,D'007'
		return
led4_red	bcf	dled,D'006'
		bsf	dled,D'007'
		return
;-------------------------------------------------
decode_digit	clrf	d_digit
		movlw	D'010'
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd10		;-

		movlw	D'000'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd00

		movlw	D'001'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd01

		movlw	D'002'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd02

		movlw	D'003'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd03

		movlw	D'004'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd04

		movlw	D'005'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd05

		movlw	D'006'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd06

		movlw	D'007'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd07

		movlw	D'008'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd08

		movlw	D'009'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd09

		movlw	D'020'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd20		;L

		movlw	D'021'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd21		;c

		movlw	D'022'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd22		;A

		movlw	D'025'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd25		;E

		movlw	D'026'
		bcf	STATUS,Z
		xorwf	d2d,0
		btfsc	STATUS,Z
		goto	dd26		;r


;

;error - just display "."
		movlw	B'00000001'
		movwf	d_digit
		return
;---------------------------------------
dd00		movlw	B'11111100'
		movwf	d_digit
		return
;---------------------------------------
dd01		movlw	B'01100000'
		movwf	d_digit
		return
;---------------------------------------
dd02		movlw	B'11011010'
		movwf	d_digit
		return
;---------------------------------------
dd03		movlw	B'11110010'
		movwf	d_digit
		return
;---------------------------------------
dd04		movlw	B'01100110'
		movwf	d_digit
		return
;---------------------------------------
dd05		movlw	B'10110110'
		movwf	d_digit
		return
;---------------------------------------
dd06		movlw	B'10111110'
		movwf	d_digit
		return
;---------------------------------------
dd07		movlw	B'11100100'
		movwf	d_digit
		return
;---------------------------------------
dd08		movlw	B'11111110'
		movwf	d_digit
		return
;---------------------------------------
dd09		movlw	B'11110110'
		movwf	d_digit
		return
;---------------------------------------
dd10		movlw	B'00000010'		;"-"
		movwf	d_digit
		return
;---------------------------------------
dd20		movlw	B'00011100'		;"L"
		movwf	d_digit
		return
;---------------------------------------
dd21		movlw	B'00011010'		;"c"
		movwf	d_digit
		return
;---------------------------------------
dd22		movlw	B'11101110'		;"A"
		movwf	d_digit
		return
;---------------------------------------
dd25		movlw	B'10011110'		;"E"
		movwf	d_digit
		return
;---------------------------------------
dd26		movlw	B'00001010'		;"r"
		movwf	d_digit
		return
;----------------------------------------------------------------------------
;------------------------- Hardware Test Routing ----------------------------
;----------------------------------------------------------------------------
hw_test		clrf	PORTD
		movlw	B'00000011'
		movwf	PORTE

		clrf	PORTB

m001		btfsc	PORTB,D'006'
		goto	c_on
;c_off
;		bsf	PORTB,D'007'		;never switch the power on during HW test
		goto	cnt001

c_on		nop
		bcf	PORTB,D'007'

cnt001		btfss	PORTE,D'003'
		call	disp01
		btfss	PORTA,D'007'
		call	disp02


		bsf	PORTC,D'000'
		bcf	PORTC,D'001'
		CALL	dr2
		bsf	PORTC,D'001'
		bcf	PORTC,D'000'
		CALL	dr2

		movlw	B'10101010'
		movwf	PORTC
		call	dr2
		movlw	B'01010101'
		movwf	PORTC
		call	dr2
		movlw	B'10101010'
		movwf	PORTC
		call	dr2
		movlw	B'01010101'
		movwf	PORTC
		call	dr2

		movlw	B'10000000'
		movwf	PORTC
		call	dr2
		movlw	B'00100000'
		movwf	PORTC
		call	dr2
		movlw	B'00001000'
		movwf	PORTC
		call	dr2
		movlw	B'00000010'
		movwf	PORTC
		call	dr2

		movlw	B'00000001'
		movwf	PORTC
		call	dr2
		movlw	B'00000100'
		movwf	PORTC
		call	dr2
		movlw	B'00010000'
		movwf	PORTC
		call	dr2
		movlw	B'01000000'
		movwf	PORTC
		call	dr2

		movlw	B'10101010'
		movwf	PORTC
		call	dr2
		movlw	B'01101010'
		movwf	PORTC
		call	dr2
		movlw	B'10011010'
		movwf	PORTC
		call	dr2
		movlw	B'10100110'
		movwf	PORTC
		call	dr2
		movlw	B'10101001'
		movwf	PORTC
		call	dr2

		clrf	PORTC

		movlw	B'11111111'
		movwf	PORTD

		movlw	B'00000110'
		movwf	PORTE
		call	dr2		

		movlw	B'00000101'
		movwf	PORTE
		call	dr2

		movlw	B'00000011'
		movwf	PORTE
		call	dr2

;digit 1
		movlw	B'00000110'
		movwf	PORTE

		movlw	B'00000010'
		movwf	PORTD
		call	dr2
		movlw	B'00000100'
		movwf	PORTD
		call	dr2
		movlw	B'00001000'
		movwf	PORTD
		call	dr2
		movlw	B'00010000'
		movwf	PORTD
		call	dr2
		movlw	B'00100000'
		movwf	PORTD
		call	dr2
		movlw	B'01000000'
		movwf	PORTD
		call	dr2
		movlw	B'10000000'
		movwf	PORTD
		call	dr2
		movlw	B'00000001'
		movwf	PORTD
		call	dr2


;digit 2
		movlw	B'00000101'
		movwf	PORTE

		movlw	B'00000010'
		movwf	PORTD
		call	dr2
		movlw	B'00000100'
		movwf	PORTD
		call	dr2
		movlw	B'00001000'
		movwf	PORTD
		call	dr2
		movlw	B'00010000'
		movwf	PORTD
		call	dr2
		movlw	B'00100000'
		movwf	PORTD
		call	dr2
		movlw	B'01000000'
		movwf	PORTD
		call	dr2
		movlw	B'10000000'
		movwf	PORTD
		call	dr2
		movlw	B'00000001'
		movwf	PORTD
		call	dr2

;digit 3
		movlw	B'00000011'
		movwf	PORTE

		movlw	B'00000010'
		movwf	PORTD
		call	dr2
		movlw	B'00000100'
		movwf	PORTD
		call	dr2
		movlw	B'00001000'
		movwf	PORTD
		call	dr2
		movlw	B'00010000'
		movwf	PORTD
		call	dr2
		movlw	B'00100000'
		movwf	PORTD
		call	dr2
		movlw	B'01000000'
		movwf	PORTD
		call	dr2
		movlw	B'10000000'
		movwf	PORTD
		call	dr2
		movlw	B'00000001'
		movwf	PORTD
		call	dr2

		clrf	PORTD

		return			;hw_test

;-------------------------------------------------------------------------
disp01		movlw	B'00000110'
		movwf	PORTE
		movlw	B'01010101'
		movwf	PORTD
		call	dr2
		btfss	PORTE,D'003'
		goto	disp01
		return

disp02		movlw	B'00000101'
		movwf	PORTE
		movlw	B'01010101'
		movwf	PORTD
		call	dr2
		btfss	PORTA,D'007'
		goto	disp02
		return
;-------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;		*********************************************
;		*               delay routine               *
;		*********************************************

delay_routine	MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return

dr2		call	delay_routine
		call	delay_routine
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX



		ORG	800

PAGE1		nop

LED_set2	nop
		PAGESEL	PAGE0
		call	LED_set
		PAGESEL	PAGE1
		return
;
power_off2	nop
		PAGESEL	PAGE0
		call	power_off
		PAGESEL	PAGE1
		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 CMOS 4001
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
CM4001		movlw	D'000'
		movwf	disp1
		movlw	D'001'
		movwf	disp2
		movlw	D'021'
		movwf	disp3			;"C" - CMOS
;four gates --> first 2 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		movwf	LED3
		movwf	LED4
;
y4001		btfss	PORTE,D'003'
		goto	y4001
;run requested?
		btfss	PORTB,D'006'
		goto	run4001
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y4001
;-------------------------------------------
run4001; (4 x 2-input NOR)
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01001100'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11001100'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'
		bsf	gate3,D'007'
		bsf	gate4,D'007'

test4001	nop

; 0 0 --> 1
; 0 1 --> 0
; 1 0 --> 0
; 1 1 --> 0


;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		btfsc	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		btfsc	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		btfsc	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'

;Gate #2/1
		bcf	PORTB,D'004'
		bcf	PORTB,D'005'
		btfss	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bsf	PORTB,D'004'
		bcf	PORTB,D'005'
		btfsc	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bcf	PORTB,D'004'
		bsf	PORTB,D'005'
		btfsc	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bsf	PORTB,D'004'
		bsf	PORTB,D'005'
		btfsc	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2

;Gate #3
		bcf	gate3,D'000'

;Gate #3/1
		bcf	PORTA,D'004'
		bcf	PORTA,D'005'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/2
		bsf	PORTA,D'004'
		bcf	PORTA,D'005'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/3
		bcf	PORTA,D'004'
		bsf	PORTA,D'005'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/4
		bsf	PORTA,D'004'
		bsf	PORTA,D'005'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3

;Gate #4
		bcf	gate4,D'000'

;Gate #4/1
		bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/2
		bsf	PORTA,D'000'
		bcf	PORTA,D'001'
		btfsc	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/3
		bcf	PORTA,D'000'
		bsf	PORTA,D'001'
		btfsc	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/4
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		btfsc	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4

		call	LED_set2
;

		btfsc	ata,D'000'		;auto test active?
		return				;yes - return


		btfss	PORTB,D'006'
		goto	test4001
		call	power_off2
		goto	CM4001
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------


;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 CMOS 4002
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
CM4002		movlw	D'000'
		movwf	disp1
		movlw	D'002'
		movwf	disp2
		movlw	D'021'
		movwf	disp3			;"C" - CMOS
;two gates --> first 2 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		clrf	LED3
		clrf	LED4
;
y4002		btfss	PORTE,D'003'
		goto	y4002
;run requested?
		btfss	PORTB,D'006'
		goto	run4002
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y4002
;-------------------------------------------
run4002; (2 x 4-input NOR)
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01100001'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11100001'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'

test4002	nop

;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'001'
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bsf	PORTB,D'001'
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfsc	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bcf	PORTB,D'001'
		bsf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfsc	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bcf	PORTB,D'001'
		bcf	PORTB,D'002'
		bsf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfsc	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/5
		bcf	PORTB,D'001'
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfsc	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/6
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfsc	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'
;Gate #2/1
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bsf	PORTA,D'001'
		bcf	PORTA,D'002'
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bcf	PORTA,D'001'
		bsf	PORTA,D'002'
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		bsf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/5
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		bcf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/6
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2

		call	LED_set2
;

		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test4002
		call	power_off2
		goto	CM4002
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------


;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 CMOS 4011
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
CM4011		movlw	D'001'
		movwf	disp1
		movlw	D'001'
		movwf	disp2
		movlw	D'021'
		movwf	disp3			;"C" - CMOS
;four gates --> first 2 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		movwf	LED3
		movwf	LED4
;
y4011		btfss	PORTE,D'003'
		goto	y4011
;run requested?
		btfss	PORTB,D'006'
		goto	run4011
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y4011
;-------------------------------------------
run4011; (4 x 2-input NAND)
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01001100'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11001100'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'
		bsf	gate3,D'007'
		bsf	gate4,D'007'

test4011	nop

; 0 0 --> 1
; 0 1 --> 1
; 1 0 --> 1
; 1 1 --> 0


;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		btfsc	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'

;Gate #2/1
		bcf	PORTB,D'004'
		bcf	PORTB,D'005'
		btfss	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bsf	PORTB,D'004'
		bcf	PORTB,D'005'
		btfss	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bcf	PORTB,D'004'
		bsf	PORTB,D'005'
		btfss	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bsf	PORTB,D'004'
		bsf	PORTB,D'005'
		btfsc	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2

;Gate #3
		bcf	gate3,D'000'

;Gate #3/1
		bcf	PORTA,D'004'
		bcf	PORTA,D'005'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/2
		bsf	PORTA,D'004'
		bcf	PORTA,D'005'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/3
		bcf	PORTA,D'004'
		bsf	PORTA,D'005'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/4
		bsf	PORTA,D'004'
		bsf	PORTA,D'005'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3

;Gate #4
		bcf	gate4,D'000'

;Gate #4/1
		bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/2
		bsf	PORTA,D'000'
		bcf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/3
		bcf	PORTA,D'000'
		bsf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/4
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		btfsc	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4

		call	LED_set2
;
		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test4011
		call	power_off2
		goto	CM4011
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 CMOS 4012
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
CM4012		movlw	D'001'
		movwf	disp1
		movlw	D'002'
		movwf	disp2
		movlw	D'021'
		movwf	disp3			;"C" - CMOS
;two gates --> first 2 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		clrf	LED3
		clrf	LED4
;
y4012		btfss	PORTE,D'003'
		goto	y4012
;run requested?
		btfss	PORTB,D'006'
		goto	run4012
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y4012
;-------------------------------------------
run4012; (2 x 4-input NAND)
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01100001'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11100001'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'

test4012	nop

;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'001'
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bcf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bsf	PORTB,D'001'
		bcf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bcf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/5
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/6
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfsc	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'
;Gate #2/1
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bcf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bsf	PORTA,D'001'
		bcf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bcf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/5
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/6
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2

		call	LED_set2
;
		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test4012
		call	power_off2
		goto	CM4012
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 CMOS 4025
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
CM4025		movlw	D'002'
		movwf	disp1
		movlw	D'005'
		movwf	disp2
		movlw	D'021'
		movwf	disp3			;"C" - CMOS
;three gates --> first 3 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		movwf	LED3
		clrf	LED4

;
y4025		btfss	PORTE,D'003'
		goto	y4025
;run requested?
		btfss	PORTB,D'006'
		goto	run4025
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y4025
;-------------------------------------------
run4025; (3 x 3-input NOR)
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01100000'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11011000'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'
		bsf	gate3,D'007'

test4025	nop


;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		bcf	PORTA,D'005'
		btfss	PORTA,D'004'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		bcf	PORTA,D'005'
		btfsc	PORTA,D'004'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		bcf	PORTA,D'005'
		btfsc	PORTA,D'004'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		bsf	PORTA,D'005'
		btfsc	PORTA,D'004'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/5
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTA,D'005'
		btfsc	PORTA,D'004'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'

;Gate #2/1
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bsf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfsc	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bcf	PORTB,D'002'
		bsf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfsc	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfsc	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/5
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfsc	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2

;Gate #3
		bcf	gate3,D'000'

;Gate #3/1
		bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/2
		bsf	PORTA,D'000'
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/3
		bcf	PORTA,D'000'
		bsf	PORTA,D'001'
		bcf	PORTA,D'002'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/4
		bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		bsf	PORTA,D'002'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/5
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3

		call	LED_set2
;
		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test4025
		call	power_off2
		goto	CM4025
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------




;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 CMOS 4030
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
CM4030		movlw	D'003'
		movwf	disp1
		movlw	D'000'
		movwf	disp2
		movlw	D'021'
		movwf	disp3			;"C" - CMOS
;four gates --> first 2 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		movwf	LED3
		movwf	LED4
;
y4030		btfss	PORTE,D'003'
		goto	y4030
;run requested?
		btfss	PORTB,D'006'
		goto	run4030
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y4030
;-------------------------------------------
run4030; (4 x 2-input OR)
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01001100'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11001100'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'
		bsf	gate3,D'007'
		bsf	gate4,D'007'

test4030	nop

; 0 0 --> 0
; 0 1 --> 1
; 1 0 --> 1
; 1 1 --> 0


;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		btfsc	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		btfsc	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'

;Gate #2/1
		bcf	PORTB,D'004'
		bcf	PORTB,D'005'
		btfsc	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bsf	PORTB,D'004'
		bcf	PORTB,D'005'
		btfss	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bcf	PORTB,D'004'
		bsf	PORTB,D'005'
		btfss	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bsf	PORTB,D'004'
		bsf	PORTB,D'005'
		btfsc	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2

;Gate #3
		bcf	gate3,D'000'

;Gate #3/1
		bcf	PORTA,D'004'
		bcf	PORTA,D'005'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/2
		bsf	PORTA,D'004'
		bcf	PORTA,D'005'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/3
		bcf	PORTA,D'004'
		bsf	PORTA,D'005'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/4
		bsf	PORTA,D'004'
		bsf	PORTA,D'005'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3

;Gate #4
		bcf	gate4,D'000'

;Gate #4/1
		bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		btfsc	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/2
		bsf	PORTA,D'000'
		bcf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/3
		bcf	PORTA,D'000'
		bsf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/4
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		btfsc	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4

		call	LED_set2
;
		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test4030
		call	power_off2
		goto	CM4030
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------


;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 CMOS 4068            !!!!!!!!!!!!!!!!!!!!
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
CM4068		movlw	D'006'
		movwf	disp1
		movlw	D'008'
		movwf	disp2
		movlw	D'021'
		movwf	disp3			;"C" - CMOS
;one gate --> first LED is used
		movlw	B'00000001'
		movwf	LED1
		clrf	LED2
		clrf	LED3
		clrf	LED4
;
y4068		btfss	PORTE,D'003'
		goto	y4068
;run requested?
		btfss	PORTB,D'006'
		goto	run4068
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y4068
;-------------------------------------------
run4068; (1 x 8-input NAND)
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01100001'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11100001'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'

test4068	nop

;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'001'
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bcf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bsf	PORTB,D'001'
		bcf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bcf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/5
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bcf	PORTB,D'004'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/6
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bcf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/7
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTA,D'001'
		bcf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/8
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bcf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/9
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/10
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate1,D'000'		;error on gate 1

		call	LED_set2
;
		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test4068
		call	power_off2
		goto	CM4068
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 CMOS 4071
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
CM4071		movlw	D'007'
		movwf	disp1
		movlw	D'001'
		movwf	disp2
		movlw	D'021'
		movwf	disp3			;"C" - CMOS
;four gates --> first 2 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		movwf	LED3
		movwf	LED4
;
y4071		btfss	PORTE,D'003'
		goto	y4071
;run requested?
		btfss	PORTB,D'006'
		goto	run4071
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y4071
;-------------------------------------------
run4071; (4 x 2-input OR)
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01001100'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11001100'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'
		bsf	gate3,D'007'
		bsf	gate4,D'007'

test4071	nop

; 0 0 --> 0
; 0 1 --> 1
; 1 0 --> 1
; 1 1 --> 1


;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		btfsc	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'

;Gate #2/1
		bcf	PORTB,D'004'
		bcf	PORTB,D'005'
		btfsc	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bsf	PORTB,D'004'
		bcf	PORTB,D'005'
		btfss	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bcf	PORTB,D'004'
		bsf	PORTB,D'005'
		btfss	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bsf	PORTB,D'004'
		bsf	PORTB,D'005'
		btfss	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2

;Gate #3
		bcf	gate3,D'000'

;Gate #3/1
		bcf	PORTA,D'004'
		bcf	PORTA,D'005'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/2
		bsf	PORTA,D'004'
		bcf	PORTA,D'005'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/3
		bcf	PORTA,D'004'
		bsf	PORTA,D'005'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/4
		bsf	PORTA,D'004'
		bsf	PORTA,D'005'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3

;Gate #4
		bcf	gate4,D'000'

;Gate #4/1
		bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		btfsc	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/2
		bsf	PORTA,D'000'
		bcf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/3
		bcf	PORTA,D'000'
		bsf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/4
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4

		call	LED_set2
;
		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test4071
		call	power_off2
		goto	CM4071
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 CMOS 4072
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
CM4072		movlw	D'007'
		movwf	disp1
		movlw	D'002'
		movwf	disp2
		movlw	D'021'
		movwf	disp3			;"C" - CMOS
;two gates --> first 2 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		clrf	LED3
		clrf	LED4
;
y4072		btfss	PORTE,D'003'
		goto	y4072
;run requested?
		btfss	PORTB,D'006'
		goto	run4072
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y4072
;-------------------------------------------
run4072; (2 x 4-input OR)
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01100001'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11100001'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'

test4072	nop

;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'001'
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfsc	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bsf	PORTB,D'001'
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bcf	PORTB,D'001'
		bsf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bcf	PORTB,D'001'
		bcf	PORTB,D'002'
		bsf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/5
		bcf	PORTB,D'001'
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/6
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'
;Gate #2/1
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bsf	PORTA,D'001'
		bcf	PORTA,D'002'
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bcf	PORTA,D'001'
		bsf	PORTA,D'002'
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		bsf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/5
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		bcf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/6
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2

		call	LED_set2
;
		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test4072
		call	power_off2
		goto	CM4072
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------


;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 CMOS 4075
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
CM4075		movlw	D'007'
		movwf	disp1
		movlw	D'005'
		movwf	disp2
		movlw	D'021'
		movwf	disp3			;"C" - CMOS
;three gates --> first 3 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		movwf	LED3
		clrf	LED4

;
y4075		btfss	PORTE,D'003'
		goto	y4075
;run requested?
		btfss	PORTB,D'006'
		goto	run4075
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y4075
;-------------------------------------------
run4075; (3 x 3-input OR)
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01100000'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11011000'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'
		bsf	gate3,D'007'

test4075	nop


;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		bcf	PORTA,D'005'
		btfsc	PORTA,D'004'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		bcf	PORTA,D'005'
		btfss	PORTA,D'004'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		bcf	PORTA,D'005'
		btfss	PORTA,D'004'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		bsf	PORTA,D'005'
		btfss	PORTA,D'004'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/5
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		bsf	PORTA,D'005'
		btfss	PORTA,D'004'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'

;Gate #2/1
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfsc	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bsf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bcf	PORTB,D'002'
		bsf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/5
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'005'
		bsf	gate2,D'000'		;error on gate 2

;Gate #3
		bcf	gate3,D'000'

;Gate #3/1
		bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/2
		bsf	PORTA,D'000'
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/3
		bcf	PORTA,D'000'
		bsf	PORTA,D'001'
		bcf	PORTA,D'002'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/4
		bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		bsf	PORTA,D'002'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/5
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3

		call	LED_set2
;
		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test4075
		call	power_off2
		goto	CM4075
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 CMOS 4081
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
CM4081		movlw	D'008'
		movwf	disp1
		movlw	D'001'
		movwf	disp2
		movlw	D'021'
		movwf	disp3			;"C" - CMOS
;four gates --> first 2 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		movwf	LED3
		movwf	LED4
;
y4081		btfss	PORTE,D'003'
		goto	y4081
;run requested?
		btfss	PORTB,D'006'
		goto	run4081
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y4081
;-------------------------------------------
run4081; (4 x 2-input AND)
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01001100'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11001100'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'
		bsf	gate3,D'007'
		bsf	gate4,D'007'

test4081	nop

; 0 0 --> 0
; 0 1 --> 0
; 1 0 --> 0
; 1 1 --> 1


;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'000'
		bcf	PORTB,D'001'
		btfsc	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bsf	PORTB,D'000'
		bcf	PORTB,D'001'
		btfsc	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bcf	PORTB,D'000'
		bsf	PORTB,D'001'
		btfsc	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'000'
		bsf	PORTB,D'001'
		btfss	PORTB,D'002'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'

;Gate #2/1
		bcf	PORTB,D'004'
		bcf	PORTB,D'005'
		btfsc	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bsf	PORTB,D'004'
		bcf	PORTB,D'005'
		btfsc	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bcf	PORTB,D'004'
		bsf	PORTB,D'005'
		btfsc	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bsf	PORTB,D'004'
		bsf	PORTB,D'005'
		btfss	PORTB,D'003'
		bsf	gate2,D'000'		;error on gate 2

;Gate #3
		bcf	gate3,D'000'

;Gate #3/1
		bcf	PORTA,D'004'
		bcf	PORTA,D'005'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/2
		bsf	PORTA,D'004'
		bcf	PORTA,D'005'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/3
		bcf	PORTA,D'004'
		bsf	PORTA,D'005'
		btfsc	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3
;Gate #3/4
		bsf	PORTA,D'004'
		bsf	PORTA,D'005'
		btfss	PORTA,D'003'
		bsf	gate3,D'000'		;error on gate 3

;Gate #4
		bcf	gate4,D'000'

;Gate #4/1
		bcf	PORTA,D'000'
		bcf	PORTA,D'001'
		btfsc	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/2
		bsf	PORTA,D'000'
		bcf	PORTA,D'001'
		btfsc	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/3
		bcf	PORTA,D'000'
		bsf	PORTA,D'001'
		btfsc	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4
;Gate #4/4
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		btfss	PORTA,D'002'
		bsf	gate4,D'000'		;error on gate 4

		call	LED_set2
;
		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test4081
		call	power_off2
		goto	CM4081
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;                 CMOS 4082
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
CM4082		movlw	D'008'
		movwf	disp1
		movlw	D'002'
		movwf	disp2
		movlw	D'021'
		movwf	disp3			;"C" - CMOS
;two gates --> first 2 LEDs are used
		movlw	B'00000001'
		movwf	LED1
		movwf	LED2
		clrf	LED3
		clrf	LED4
;
y4082		btfss	PORTE,D'003'
		goto	y4082
;run requested?
		btfss	PORTB,D'006'
		goto	run4082
;some other button pressed?
		btfss	PORTE,D'003'
		return				;back to main program
		goto	y4082
;-------------------------------------------
run4082; (2 x 4-input AND)
;1. init I/O
		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01100001'	;PORTB = input/output
		MOVWF	TRISB
		MOVLW	B'11100001'	;PORTA = input/output
		MOVWF	TRISA
		bcf	STATUS,RP0	;Switch to register bank 0
;2. switch on power
		bsf	PORTB,D'007'

;3. execute test

		clrf	gate1
		clrf	gate2
		clrf	gate3
		clrf	gate4

		bsf	gate1,D'007'
		bsf	gate2,D'007'

test4082	nop

;Gate #1
		bcf	gate1,D'000'

;Gate #1/1
		bcf	PORTB,D'001'
		bcf	PORTB,D'002'
		bcf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfsc	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/2
		bcf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfsc	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/3
		bsf	PORTB,D'001'
		bcf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfsc	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/4
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bcf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfsc	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/5
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bcf	PORTB,D'004'
		btfsc	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1
;Gate #1/6
		bsf	PORTB,D'001'
		bsf	PORTB,D'002'
		bsf	PORTB,D'003'
		bsf	PORTB,D'004'
		btfss	PORTB,D'000'
		bsf	gate1,D'000'		;error on gate 1

;Gate #2
		bcf	gate2,D'000'
;Gate #2/1
		bcf	PORTA,D'001'
		bcf	PORTA,D'002'
		bcf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/2
		bcf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/3
		bsf	PORTA,D'001'
		bcf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/4
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bcf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/5
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bcf	PORTA,D'004'
		btfsc	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2
;Gate #2/6
		bsf	PORTA,D'001'
		bsf	PORTA,D'002'
		bsf	PORTA,D'003'
		bsf	PORTA,D'004'
		btfss	PORTA,D'000'
		bsf	gate2,D'000'		;error on gate 2

		call	LED_set2
;
		btfsc	ata,D'000'		;auto test active?
		return				;yes - return

		btfss	PORTB,D'006'
		goto	test4082
		call	power_off2
		goto	CM4082
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------



		END